import sys
import os
from progress.bar import Bar

from path import Path
from picture import Picture
from bulk_page import BulkPage
from bulk_album import BulkAlbum
import argparse
import config

from jinja2 import Environment, FileSystemLoader

env = Environment(loader=FileSystemLoader('src/templates'))
home_template = env.get_template('home.jinja')

def argument_parsing():
    parser = argparse.ArgumentParser()
    parser.add_argument("site_path", help="the folder of your site MUST BE ALWAYS BEFORT PARAMETERS", type=str)
    parser.add_argument("-r", '--regen', help="Regenerate (t)thumb (s)small", type=str)
    args = parser.parse_args()
    return args

def scan_pages(folders: list[Path]) -> list[BulkPage]:
    pages: list[BulkPage] = []
    prev: BulkPage = None
    with Bar("Scanning Pages...", max=len(folders)) as bar:
        for folder in folders:
            files: list[Path] = folder.get_files()

            page: BulkPage = BulkPage(folder, folder.get_name(), prev=prev)

            raw: Path = Path(folder, folder.get_name() + ".NEF")
            id: int = 0
            for file in files:
                if file.get_name().endswith(".png"):
                    page.add_picture(Picture(file, id, page=page, raw=raw))
                    id += 1

            if len(page.get_pictures()) == 0:
                bar.next()
                continue

            pages.append(page)
            if prev is not None:
                prev.next = page
            prev = page

            bar.next()
    pages[-1].next = pages[0]
    pages[0].prev = pages[-1]
    return pages

def create_pages(pages: list[BulkPage]) -> None:
    with Bar("Generating Pages...", max=len(pages)) as bar:
        for page in pages:
            page.create()
            bar.next()

def scan_albums(pages: list[BulkPage], albums_path: Path) -> list[BulkAlbum]:
    albums: dict[str, BulkAlbum] = {}
    with Bar("Scanning pages...", max=len(pages)) as bar:
        for page in pages:
            for picture in page.get_pictures():
                for album_name in picture.get_albums_name():
                    album: BulkAlbum | None = albums.get(album_name)
                    if (album is None):
                        album = BulkAlbum(album_name, albums_path)
                        albums.update({album_name: album})
                    picture.albums.append(album)
                    album.add_picture(picture)
            bar.next()
    return (albums.values())

def create_albums(albums: list[BulkAlbum]) -> None:
    with Bar("Generating albums...", max=len(albums)) as bar:
        for album in albums:
            album.create()
            bar.next()

def gen_bulk(bulk_path: Path):
    album_path: Path = Path(bulk_path, "albums")

    pages: list[BulkPage] = scan_pages(bulk_path.get_dirs())
    albums: list[BulkAlbum] = scan_albums(pages, album_path)

    if config.CREATE_GENERAL_ALBUM:
        for album in albums:
            if (album.name == "general"):
                album.path = Path(bulk_path, "general.html")

    Path("./src/templates/bulk_page.css").copy_to(Path(bulk_path, "bulk_page.css"))
    create_pages(pages)

    Path("./src/templates/bulk_album.css").copy_to(Path(bulk_path, "bulk_album.css"))
    if (not album_path.exist()):
        album_path.create()
    create_albums(albums)


def regen(bulk_path: Path, is_thumb: bool, is_small: bool):
    pages: list[BulkPage] = scan_pages(bulk_path.get_dirs())
    with Bar("Generating albums...", max=len(pages)) as bar:
        for page in pages:
            for picture in page.get_pictures():
                if is_thumb:
                    picture.gen_thumb()
                if is_small:
                    picture.gen_small()
            bar.next()
    
def gen_home(site_path: Path) -> None:
    home_path = Path(site_path, "index.html")

    with open(home_path.get_absolute_path(), "w") as f:
        f.write(home_template.render())



def main():
    args = argument_parsing()
    site_path: Path = Path(args.site_path)
    bulk_path: Path = Path(site_path, "bulk/")
    if args.regen is not None:
        regen(bulk_path, 't' in args.regen, 's' in args.regen)
    gen_bulk(bulk_path)
    gen_home(site_path)


if __name__ == "__main__":
    main()
